==> libro-codemirror-0.1.7.tgz: package/es/indentation-markers/map.js <==
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
import { indentationMarkerConfig } from "./config.js";
import { getCurrentLine, numColumns } from "./utils.js";
/**
 * Indentation map for a set of lines.
 *
 * This map will contain the indentation for lines that are not a part of the given set,
 * but this is because calculating the indentation for those lines was necessary to
 * calculate the indentation for the lines provided to the constructor.
 *
 * @see {@link IndentEntry}
 */
export var IndentationMap = /*#__PURE__*/function () {
  /** The {@link EditorState} indentation is derived from. */

  /** The set of lines that are used as an entrypoint. */

  /** The internal mapping of line numbers to {@link IndentEntry} objects. */

  /** The width of the editor's indent unit. */

  /** The type of indentation to use (terminate at end of scope vs last non-empty line in scope) */

  /**
   * @param lines - The set of lines to get the indentation map for.
   * @param state - The {@link EditorState} to derive the indentation map from.
   * @param unitWidth - The width of the editor's indent unit.
   * @param markerType - The type of indentation to use (terminate at end of scope vs last line of code in scope)
   */
  function IndentationMap(lines, state, unitWidth, markerType) {
    _classCallCheck(this, IndentationMap);
    this.lines = lines;
    this.state = state;
    this.map = new Map();
    this.unitWidth = unitWidth;
    this.markerType = markerType;
    var _iterator = _createForOfIteratorHelper(this.lines),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var line = _step.value;
        this.add(line);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
    if (this.state.facet(indentationMarkerConfig).highlightActiveBlock) {
      this.findAndSetActiveLines();
    }
  }

  /**
   * Checks if the indentation map has an entry for the given line.
   *
   * @param line - The {@link Line} or line number to check for.
   */
  _createClass(IndentationMap, [{
    key: "has",
    value: function has(line) {
      return this.map.has(typeof line === 'number' ? line : line.number);
    }

    /**
     * Returns the {@link IndentEntry} for the given line.
     *
     * Note that this function will throw an error if the line does not exist in the map.
     *
     * @param line - The {@link Line} or line number to get the entry for.
     */
  }, {
    key: "get",
    value: function get(line) {
      var entry = this.map.get(typeof line === 'number' ? line : line.number);
      if (!entry) {
        throw new Error('Line not found in indentation map');
      }
      return entry;
    }

    /**
     * Sets the {@link IndentEntry} for the given line.
     *
     * @param line - The {@link Line} to set the entry for.
     * @param col - The visual beginning whitespace width of the line.
     * @param level - The indentation level of the line.
     */
  }, {
    key: "set",
    value: function set(line, col, level) {
      var empty = !line.text.trim().length;
      var entry = {
        line: line,
        col: col,
        level: level,
        empty: empty
      };
      this.map.set(entry.line.number, entry);
      return entry;
    }

    /**
     * Adds a line to the indentation map.
     *
     * @param line - The {@link Line} to add to the map.
     */
  }, {
    key: "add",
    value: function add(line) {
      if (this.has(line)) {
        return this.get(line);
      }

      // empty lines continue their indentation from surrounding lines
      if (!line.length || !line.text.trim().length) {
        // the very first line, if empty, is just ignored and set as a 0 indent level
        if (line.number === 1) {
          return this.set(line, 0, 0);
        }

        // if we're at the end, we'll just use the previous line's indentation
        if (line.number === this.state.doc.lines) {
          var _prev = this.closestNonEmpty(line, -1);
          return this.set(line, 0, _prev.level);
        }
        var prev = this.closestNonEmpty(line, -1);
        var next = this.closestNonEmpty(line, 1);

        // if the next line ends the block and the marker type is not set to codeOnly,
        // we'll just use the previous line's indentation
        if (prev.level >= next.level && this.markerType !== 'codeOnly') {
          return this.set(line, 0, prev.level);
        }

        // having an indent marker that starts from an empty line looks weird
        if (prev.empty && prev.level === 0 && next.level !== 0) {
          return this.set(line, 0, 0);
        }

        // if the next indentation level is greater than the previous,
        // we'll only increment up to the next indentation level. this prevents
        // a weirdly "backwards propagating" indentation.
        if (next.level > prev.level) {
          return this.set(line, 0, prev.level + 1);
        }

        // else, we default to the next line's indentation
        return this.set(line, 0, next.level);
      }
      var col = numColumns(line.text, this.state.tabSize);
      var level = Math.floor(col / this.unitWidth);
      return this.set(line, col, level);
    }

    /**
     * Finds the closest non-empty line, starting from the given line.
     *
     * @param from - The {@link Line} to start from.
     * @param dir - The direction to search in. Either `1` or `-1`.
     */
  }, {
    key: "closestNonEmpty",
    value: function closestNonEmpty(from, dir) {
      var lineNo = from.number + dir;
      while (dir === -1 ? lineNo >= 1 : lineNo <= this.state.doc.lines) {
        if (this.has(lineNo)) {
          var entry = this.get(lineNo);
          if (!entry.empty) {
            return entry;
          }
        }

        // we can check if the line is empty, if it's not, we can
        // just create a new entry for it and return it.
        // this prevents us from hitting the beginning/end of the document unnecessarily.

        var _line = this.state.doc.line(lineNo);
        if (_line.text.trim().length) {
          var col = numColumns(_line.text, this.state.tabSize);
          var level = Math.floor(col / this.unitWidth);
          return this.set(_line, col, level);
        }
        lineNo += dir;
      }

      // if we're here, we didn't find anything.
      // that means we're at the beginning/end of the document,
      // and the first/last line is empty.

      var line = this.state.doc.line(dir === -1 ? 1 : this.state.doc.lines);
      return this.set(line, 0, 0);
    }

    /**
     * Finds the state's active block (via the current selection) and sets all
     * the active indent level for the lines in the block.
     */
  }, {
    key: "findAndSetActiveLines",
    value: function findAndSetActiveLines() {
      var currentLine = getCurrentLine(this.state);
      if (!this.has(currentLine)) {
        return;
      }
      var current = this.get(currentLine);

      // check if the current line is starting a new block, if yes, we want to
      // start from inside the block.
      if (this.has(current.line.number + 1)) {
        var next = this.get(current.line.number + 1);
        if (next.level > current.level) {
          current = next;
        }
      }

      // same, but if the current line is ending a block
      if (this.has(current.line.number - 1)) {
        var prev = this.get(current.line.number - 1);
        if (prev.level > current.level) {
          current = prev;
        }
      }
      if (current.level === 0) {
        return;
      }
      current.active = current.level;
      var start;
      var end;

      // iterate to the start of the block
      for (start = current.line.number; start > 1; start--) {
        if (!this.has(start - 1)) {
          continue;
        }
        var _prev2 = this.get(start - 1);
        if (_prev2.level < current.level) {
          break;
        }
        _prev2.active = current.level;
      }

      // iterate to the end of the block
      for (end = current.line.number; end < this.state.doc.lines; end++) {
        if (!this.has(end + 1)) {
          continue;
        }
        var _next = this.get(end + 1);
        if (_next.level < current.level) {
          break;
        }
        _next.active = current.level;
      }
    }
  }]);
  return IndentationMap;
}();

==> libro-codemirror-0.1.13.tgz: package/es/indentation-markers/map.js <==
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : String(i); }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
import { indentationMarkerConfig } from "./config.js";
import { getCurrentLine, numColumns } from "./utils.js";
/**
 * Indentation map for a set of lines.
 *
 * This map will contain the indentation for lines that are not a part of the given set,
 * but this is because calculating the indentation for those lines was necessary to
 * calculate the indentation for the lines provided to the constructor.
 *
 * @see {@link IndentEntry}
 */
export var IndentationMap = /*#__PURE__*/function () {
  /** The {@link EditorState} indentation is derived from. */

  /** The set of lines that are used as an entrypoint. */

  /** The internal mapping of line numbers to {@link IndentEntry} objects. */

  /** The width of the editor's indent unit. */

  /** The type of indentation to use (terminate at end of scope vs last non-empty line in scope) */

  /**
   * @param lines - The set of lines to get the indentation map for.
   * @param state - The {@link EditorState} to derive the indentation map from.
   * @param unitWidth - The width of the editor's indent unit.
   * @param markerType - The type of indentation to use (terminate at end of scope vs last line of code in scope)
   */
  function IndentationMap(lines, state, unitWidth, markerType) {
    _classCallCheck(this, IndentationMap);
    this.lines = lines;
    this.state = state;
    this.map = new Map();
    this.unitWidth = unitWidth;
    this.markerType = markerType;
    var _iterator = _createForOfIteratorHelper(this.lines),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var line = _step.value;
        this.add(line);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
    if (this.state.facet(indentationMarkerConfig).highlightActiveBlock) {
      this.findAndSetActiveLines();
    }
  }

  /**
   * Checks if the indentation map has an entry for the given line.
   *
   * @param line - The {@link Line} or line number to check for.
   */
  _createClass(IndentationMap, [{
    key: "has",
    value: function has(line) {
      return this.map.has(typeof line === 'number' ? line : line.number);
    }

    /**
     * Returns the {@link IndentEntry} for the given line.
     *
     * Note that this function will throw an error if the line does not exist in the map.
     *
     * @param line - The {@link Line} or line number to get the entry for.
     */
  }, {
    key: "get",
    value: function get(line) {
      var entry = this.map.get(typeof line === 'number' ? line : line.number);
      if (!entry) {
        throw new Error('Line not found in indentation map');
      }
      return entry;
    }

    /**
     * Sets the {@link IndentEntry} for the given line.
     *
     * @param line - The {@link Line} to set the entry for.
     * @param col - The visual beginning whitespace width of the line.
     * @param level - The indentation level of the line.
     */
  }, {
    key: "set",
    value: function set(line, col, level) {
      var empty = !line.text.trim().length;
      var entry = {
        line: line,
        col: col,
        level: level,
        empty: empty
      };
      this.map.set(entry.line.number, entry);
      return entry;
    }

    /**
     * Adds a line to the indentation map.
     *
     * @param line - The {@link Line} to add to the map.
     */
  }, {
    key: "add",
    value: function add(line) {
      if (this.has(line)) {
        return this.get(line);
      }

      // empty lines continue their indentation from surrounding lines
      if (!line.length || !line.text.trim().length) {
        // the very first line, if empty, is just ignored and set as a 0 indent level
        if (line.number === 1) {
          return this.set(line, 0, 0);
        }

        // if we're at the end, we'll just use the previous line's indentation
        if (line.number === this.state.doc.lines) {
          var _prev = this.closestNonEmpty(line, -1);
          return this.set(line, 0, _prev.level);
        }
        var prev = this.closestNonEmpty(line, -1);
        var next = this.closestNonEmpty(line, 1);

        // if the next line ends the block and the marker type is not set to codeOnly,
        // we'll just use the previous line's indentation
        if (prev.level >= next.level && this.markerType !== 'codeOnly') {
          return this.set(line, 0, prev.level);
        }

        // having an indent marker that starts from an empty line looks weird
        if (prev.empty && prev.level === 0 && next.level !== 0) {
          return this.set(line, 0, 0);
        }

        // if the next indentation level is greater than the previous,
        // we'll only increment up to the next indentation level. this prevents
        // a weirdly "backwards propagating" indentation.
        if (next.level > prev.level) {
          return this.set(line, 0, prev.level + 1);
        }

        // else, we default to the next line's indentation
        return this.set(line, 0, next.level);
      }
      var col = numColumns(line.text, this.state.tabSize);
      var level = Math.floor(col / this.unitWidth);
      return this.set(line, col, level);
    }

    /**
     * Finds the closest non-empty line, starting from the given line.
     *
     * @param from - The {@link Line} to start from.
     * @param dir - The direction to search in. Either `1` or `-1`.
     */
  }, {
    key: "closestNonEmpty",
    value: function closestNonEmpty(from, dir) {
      var lineNo = from.number + dir;
      while (dir === -1 ? lineNo >= 1 : lineNo <= this.state.doc.lines) {
        if (this.has(lineNo)) {
          var entry = this.get(lineNo);
          if (!entry.empty) {
            return entry;
          }
        }

        // we can check if the line is empty, if it's not, we can
        // just create a new entry for it and return it.
        // this prevents us from hitting the beginning/end of the document unnecessarily.

        var _line = this.state.doc.line(lineNo);
        if (_line.text.trim().length) {
          var col = numColumns(_line.text, this.state.tabSize);
          var level = Math.floor(col / this.unitWidth);
          return this.set(_line, col, level);
        }
        lineNo += dir;
      }

      // if we're here, we didn't find anything.
      // that means we're at the beginning/end of the document,
      // and the first/last line is empty.

      var line = this.state.doc.line(dir === -1 ? 1 : this.state.doc.lines);
      return this.set(line, 0, 0);
    }

    /**
     * Finds the state's active block (via the current selection) and sets all
     * the active indent level for the lines in the block.
     */
  }, {
    key: "findAndSetActiveLines",
    value: function findAndSetActiveLines() {
      var currentLine = getCurrentLine(this.state);
      if (!this.has(currentLine)) {
        return;
      }
      var current = this.get(currentLine);

      // check if the current line is starting a new block, if yes, we want to
      // start from inside the block.
      if (this.has(current.line.number + 1)) {
        var next = this.get(current.line.number + 1);
        if (next.level > current.level) {
          current = next;
        }
      }

      // same, but if the current line is ending a block
      if (this.has(current.line.number - 1)) {
        var prev = this.get(current.line.number - 1);
        if (prev.level > current.level) {
          current = prev;
        }
      }
      if (current.level === 0) {
        return;
      }
      current.active = current.level;
      var start;
      var end;

      // iterate to the start of the block
      for (start = current.line.number; start > 1; start--) {
        if (!this.has(start - 1)) {
          continue;
        }
        var _prev2 = this.get(start - 1);
        if (_prev2.level < current.level) {
          break;
        }
        _prev2.active = current.level;
      }

      // iterate to the end of the block
      for (end = current.line.number; end < this.state.doc.lines; end++) {
        if (!this.has(end + 1)) {
          continue;
        }
        var _next = this.get(end + 1);
        if (_next.level < current.level) {
          break;
        }
        _next.active = current.level;
      }
    }
  }]);
  return IndentationMap;
}();
